/*!
    \file   df_setup_lists.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Deklarationen der Funktionen für die Arbeit mit den Setuplisten.
*/
#ifndef DF_SETUP_LISTS_H
#define DF_SETUP_LISTS_H

#include "df_defines.h"

/*! 
    \brief Setzen des Listenbuffers fuer eine Liste.

    Die Listen muessen ihrer Reichenfolge nach (0 .. n) in den DLL internen 
    Buffer uebertragen werden.

    \param connectionId Kommunikationskanalnummer.
    \param number Tabellennummer.
    \param recordCount Anzahl der Zeilen, Datensaetze im Block.
    \param size Anzahl Byte im Block. Brechnet sich aus (Anzahl Datensaetzte    Datensatzlaenge).
    \param buffer Zeiger auf Bytearray, welches die Daten enthaelt.
    \param unused_setTo0 Kann immer mit 0 uebergeben werden. (Kompatibel mit Timeboy)

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.
*/
DF_API_EXTERN int DF_API_CALL DFCMakeListe( int connectionId, int number, int recordCount, int size, unsigned char *buffer, int unused_setTo0 );


/*! 
    \brief Uebertragen der Listendaten zum Geraet.

    Es wird versucht, die zuvor mit DFCMakeListe() importierten Listendaten in 
    das Geraet zu uebertragen.

    \param connectionId Kommunikationskanalnummer.
    \param deviceAddress Busnummer des anzusprechenden Geraetes.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.
*/
DF_API_EXTERN int DF_API_CALL DFCLoadListen( int connectionId, int deviceAddress, int *errorNumber );


/*! 
    \brief Buffer fuer Listendaten loeschen.

    Loescht in der DLL den Buffer fuer Listendaten. Die zuvor schon importierten 
    Listen waehren nach Ausfuehrung wiederholt zu importieren.

    \param connectionId Kommunikationskanalnummer.

    \return Keine.
*/
DF_API_EXTERN void DF_API_CALL DFCClrListenBuffer( int connectionId );

#if (DF_API_VERSION >= 1)


/*! 
    \brief Tabellenbeschreibungen fuer Listen aus Geraet lesen.

    Es wird versucht die Tabellenbeschreibungen aus dem Geraet zu lesen. Hiernach koennen   
    mit den Funktionen DFCListBCnt(), DFCListBDatensatz() und DFCListBFeld() auf den ermittelten 
    Daten gearbeitet werden.

    \param connectionId Kommunikationskanalnummer.
    \param deviceAddress Busnummer des anzusprechenden Geraetes.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 1.1.i enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCLoadListenbeschreibung( int connectionId, int deviceAddress, int *errorNumber );


/*! 
    \brief Anzahl der Tabellenbeschreibungen fuer Listen ermitteln.

    Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Daten, wird 
    die Anzahl der definierten Tabellen zurueckgegeben.

    \param connectionId Kommunikationskanalnummer.

    \return Anzahl der mit DFCLoadListenbeschreibung() gelesenen Tabellendefinitionen.

    \version Ab DLL - Version 1.1.i enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCListBCnt( int connectionId );


/*! 
    \brief Grunddaten einer Tabellenbeschreibungen fuer Listen ermitteln.

    Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Daten, werden 
    die Grunddaten zu einer Tabellen zurueckgegeben.

    \param connectionId Kommunikationskanalnummer.
    \param number Tabellennummer im Bereich von (0 - (DFCListBCnt() - 1)).
    \param name Zeiger auf Speicherbereich fuer Tabellenname. (Mindestens 17 Zeichen).
    \param fieldCount Zeiger auf Variable, welche die Feldanzahl der Tabelle erhaelt.
    \param unused_setTo0 Zeiger auf Variable, welche immer eine 0 erhaelt. (Kompatible zu Timeboy)

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden. (Tabelle nicht definiert!)
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 1.1.i enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCListBDatensatz( int connectionId, int number, unsigned char *name, int *fieldCount, int *unused_setTo0 );


/*! 
    \brief Grunddaten eines Tabellenfeldes fuer Listen ermitteln.

    Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Daten, werden 
    die Grunddaten zu einem Tabellenfeld zurueckgegeben.

    \param connectionId Kommunikationskanalnummer.
    \param listBNumber Tabellennummer im Bereich von (0 - (DFCListBCnt() - 1)).
    \param number Feldnummer.
    \param name Zeiger auf Speicherbereich fuer Feldname. (Mindestens 17 Zeichen).
    \param type Zeiger auf Variable, welche den Feldtyp erhaelt.
    \param size Zeiger auf Variable, welche die definierte Laenge des Feldes erhaelt,
                    in dieser ist auch das Nullterminierungszeichen enthalten.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden. (Tabelle nicht definiert!)
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 1.1.i enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCListBFeld( int connectionId, int listBNumber, int number, unsigned char *name, int *type, int *size );

#endif

#endif
