/**
 * @file  FileTouchKeys.cpp
 * @brief Implementierung der Klasse FileTouchKeys.
 */
//////////////////////////////////////////////////////////////////////

#include "filetouchkeys.h"
#include <memory.h>
#include <stdio.h>

FileTouchKeys::FileTouchKeys()
{
	// CRC32 Initialisierung
	InitCRC32Table();

	// Ausgangseinstellung
	resetError();
	reset();
}

FileTouchKeys::~FileTouchKeys()
{
}

void FileTouchKeys::reset()
{
	// Basis Reset
	FileIntelHex::reset();

	// Member initialisieren
	
	// Groessen setzen
	m_sizeData   = 0;

	// Speicher fuer TouchKeys loeschen
	memset(m_memTouchKeys, 0x00, MAX_MEM_TOUCHKEYS);

	// Hilfsstrukturen aufsetzen
	m_pTouchKeysNode   = (TOUCHKEYNODE*)m_memTouchKeys;
	m_pTouchKeysNode->version = 1;
	m_pTouchKeysNode->cnt     = 0;
	m_pTouchKeysNode->temp[0] = 0;
	m_pTouchKeysNode->temp[1] = 0;
	m_pTouchKeysNode->ptrTouchkeyArray = sizeof(TOUCHKEYNODE);
}

void FileTouchKeys::Init(unsigned char count, unsigned char deviceNumber)
{
	int i, j;

	// Speicher fuer TouchKeyDaten loeschen
	memset(m_memTouchKeys, 0x00, MAX_MEM_TOUCHKEYS);

	// 1.
	// Zeichensaetze init
	// Grundstruktur aufsetzen
	m_pTouchKeysNode->version = 1;
	m_pTouchKeysNode->cnt     = count;
	m_pTouchKeysNode->temp[0] = deviceNumber;
	m_pTouchKeysNode->temp[1] = 0;
	m_pTouchKeysNode->ptrTouchkeyArray = sizeof(TOUCHKEYNODE);

	// Zeigerarray und Touchkeysstrukturzeiger initialisieren
	i = m_pTouchKeysNode->cnt;
	for (j=0; j < i; j++) {
		
		// Zeigereintrag auf Touchkeystruktur
		*(unsigned int*)&m_memTouchKeys[m_pTouchKeysNode->ptrTouchkeyArray + j * sizeof(unsigned int)] = 
			m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof(unsigned int) + j * sizeof(TOUCHKEY);

		m_pTouchKeys[j] = (TOUCHKEY*)&m_memTouchKeys[m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof(unsigned int) + j * sizeof(TOUCHKEY)];
	}

	// Variable fuer Groessenangabe
	m_sizeData = sizeof(TOUCHKEYNODE) + m_pTouchKeysNode->cnt * sizeof(unsigned int) + m_pTouchKeysNode->cnt * sizeof(TOUCHKEY);
}

int FileTouchKeys::SetTouchKey(int idx, TOUCHKEY tk)
{
	if (idx >= m_pTouchKeysNode->cnt)
	{
		return -1;
	}

	*m_pTouchKeys[idx] = tk;
	return idx;
}

TOUCHKEY FileTouchKeys::GetTouchKey(int idx)
{
	TOUCHKEY tk;
	memset(&tk, 0, sizeof(TOUCHKEY));

	if (idx >= m_pTouchKeysNode->cnt)
	{
		return tk;
	}

	return *m_pTouchKeys[idx];
}

int FileTouchKeys::Load(const char *fileName)
{
	// Lesen der Hexdatei
	FILE *fp;
	char data[0x400];

	// Fehlerobjekt zuruecksetzen
	resetError();

	// Umgebung zuruecksetzen
	reset();

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		setErrorState( FileOpen );
		setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
		return 0;
	}

	// Daten eintragen
	unsigned char type;
	unsigned short address;

	while(!feof(fp) && (errorState() == None)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			address = GetAddress(data);
			type    = GetType(data);

			switch (type) {

			case TypeTouchkeyData:
				m_sizeData += GetData(data, &m_memTouchKeys[address]);
				break;

			case TypeEOF:
				break;
			}
		}
	}
	fclose(fp);	

	// Member durch gelesene Daten initialisieren und pruefen
	if (m_sizeData) {
		
		int i, j;

		// 1.
		// Touchkeys pruefen

		// Zeigerarray und Touchkeystrukturzeiger initialisieren
		i = m_pTouchKeysNode->cnt;
		if (i > MAX_TOUCHKEYS) {

			// Fehler zuweisen
			setErrorState( Format );
			setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
			return 0;
		}

		for (j=0; j < i; j++) {
			
			m_pTouchKeys[j] = (TOUCHKEY*)&m_memTouchKeys[m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof(unsigned int) + j * sizeof(TOUCHKEY)];
		}

	} else {

		// Fehler zuweisen
		setErrorState( NoData );
		setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
		return 0;
	}

	return 1;
}



int FileTouchKeys::Save(const char *fileName, unsigned char append /*= 0*/)
{
	FILE *fp;
	char szData[0x400];

	// Fehlerobjekt zuruecksetzen
	resetError();

	// vorhandene Datei anlegen
	if (!append) {
		if ((fp = fopen(fileName, "w")) == (FILE*)0) {
			// Oeffnen der Datei gescheitert
			setErrorState( FileOpen );
			setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
			return 0;
		}
		fclose(fp);
	}

	// Daten der TouchKeys sichern
	if (!SaveTouchKeys(fileName, 1)) {

		return 0;
	}

	if (!append) {
		// Endezeile in Datei eintragen
		if ((fp = fopen(fileName, "a")) == (FILE*)0) {
			// Oeffnen der Datei gescheitert
			setErrorState( FileOpen );
			setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
			return 0;
		}

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			// Fehler zuweisen
			setErrorState( RecordEndMake );
			setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
			return 0;
		}

		fprintf(fp, "%s\n", szData);
		fclose(fp);
	}

	return 1;
}

int FileTouchKeys::SaveTouchKeys(const char *fileName, unsigned char append /*= 0*/)
{
	FILE *fp;
	char szData[0x400];

	if ((fp = fopen(fileName, (append) ? "a" : "w")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		setErrorState( FileOpen );
		setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
		return 0;
	}

	// Daten der TouchKeys eintragen
	unsigned int ulPos;
	for (ulPos=0; ulPos < m_sizeData; ulPos += maxDataSize()) {

		if (!MakeTypedDataLine((unsigned short)ulPos, TypeTouchkeyData, 
			&m_memTouchKeys[ulPos], (unsigned char)(((ulPos + maxDataSize()) < m_sizeData) ? maxDataSize() : m_sizeData - ulPos), szData)) {
			
			// Fehler zuweisen
			setErrorState( RecordMake );
			setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
			break;
		}

		fprintf(fp, "%s\n", szData);
	}

	// Abbruch nach Fehler
	if (errorState() != None) {

		fclose(fp);
		return 0;
	}

	if (!append) {

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			// Fehler zuweisen
			setErrorState( RecordEndMake );
			setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
			return 0;
		}

		fprintf(fp, "%s\n", szData);
	}

	fclose(fp);

	return 1;
}

